% This is a spacecraft attitude dynamics simulator developed by Patchedconics, LLC
% The simulation test Patchedconics estimator by calling it in the MATLAB code

function MS()
    
    clear all   % Clear Workspace
    close all   % Close All Windows
    clc         % Clear Command Window
    
    tic
    
    %% INFORMATION TO BE SET BY THE USER ----------------------------------
    
    % Time intervals

    dts = 0.1;          % Simulator                         [s]
    dte = 1;            % Estimator                         [s]

    N   = 2123;         % Simulation time                   [s]

    n   = N/dts;        % Simulation steps


    % Other variables

    B   = 1200;         % Estimator stored information      [s]

    ST  = 0;            % Steps stored by estimator before  [s]
                        % first estimation starts  

    IL  = 1;            % Number of times "I" is estimated  [s]
                        % before "L" updates                

    f   = 2000;         % Camera sensor witdth and heigth   [px]
                        % DO NOT MAKE LARGER THAN 2000
    % Masses

    M       = 500;      % Spacecraft main body mass         [kg]
    Msap    = 30;       % Solar array panel mass            [kg]

    perr    = 0;        % Pixel uncertainty                 [px]

    point_animation = 1;    % Animation of the projected markers 1 means 
                            % animation will show
                            
    surfaces        = 1;    % (1 or 2) Number of surfaces on which Makers
                            % are distributed

    estimation      = 1;    % Estimation. 1 means estimation is performed

    prints          = 0;    % Print info to commnd line (1 means yes)


    % Dimensions of Spacecraft and SAPs (divided by 2)

    a   = 6;    b   = 4;    c   = 4;
    a   = a/2;  b   = b/2;  c   = c/2; 

    d   = 1;    e   = 12;   g   = 5; 
    d   = d/2;  e   = e/2;  g   = g/2;

    theta = -pi()/6;          % Angle of solar panels             [rad]
                        % (see MESCs.png) pi()/6


    % Position and Velocity of observer w.r.t. the target

    X   = 0;    Y   = 0;    Z   = 16;   % Position          [m]
    pos = [X; Y; Z];

    VX  = 0;    VY  = 0;    VZ  = 0;    % Velocity          [m s-1]
    vel = [VX;VY;VZ];


    % Spacecraft Initial Euler angles and Rotation speed

    % phi1    = 0.28674538575; phi2 = 0.28674538575; phi3 = 0.78539816339; % [rad]
    phi1    = 0; phi2 = 0; phi3 = -pi/4; % [rad]
    phi     = [phi1;phi2;phi3];

    % om1     = 0;  om2 = 0.02759607851610028;  om3 = 0.08278823554830084;         % [rad s-1]
    om1     = 0.02;  om2 = 0.12;  om3 = 0.12;         % [rad s-1]
    omega   = [om1;om2;om3];

    %% END OF MODIFIABLE VARIABLES ----------------------------------------
    
    % Paths required for the files to be exchanged
    % between simulator and estimator
    cdc     = pwd;
    
    if surfaces == 1
        if ismac
            ptME    = fullfile(cdc,'ME','1','macOS','ME');
            ptNP    = fullfile(cdc,'ME','1','macOS','estdata.npz');
        else
            ptME    = fullfile(cdc,"ME","1","Windows","ME.exe");
            ptNP    = fullfile(cdc,"ME","1","Windows","estdata.npz");
        end
    elseif surfaces == 2
        if ismac
            ptME    = fullfile(cdc,'ME','2','macOS','ME');
            ptNP    = fullfile(cdc,'ME','2','macOS','estdata.npz');
        else
            ptME    = fullfile(cdc,"ME","2","Windows","ME.exe");
            ptNP    = fullfile(cdc,"ME","2","Windows","estdata.npz");
        end
    end
    
    ptTE    = fullfile(cdc,'data','toestimator.csv');
    ptTS    = fullfile(cdc,'data','tosimulator.csv');
    ptSP    = fullfile(cdc,'data',      'shape.csv');
    ptRE    = fullfile(cdc,'results', 'results.csv');
    ptREf   = fullfile(cdc,'results', 'results.png');

    ptDT    = fullfile(cdc,'data');
    ptRR    = fullfile(cdc,'results');
    
    paths   = {cdc; ptME; ptTE; ptTS; ptSP; ptRE; ptREf; ptNP; ptDT; ptRR};

    % Defining dimensions, position and orientation of the SAPs

    SAP1 =  [g,   0,    0, 1;
             g, 2*e,    0, 1;
             g, 2*e, -2*d, 1;
             g,   0, -2*d, 1;
            -g,   0,    0, 1;
            -g, 2*e,    0, 1;
            -g, 2*e, -2*d, 1;
            -g,   0, -2*d, 1;
             0,   e,   -d, 1]; % Vertices of SAP

    SAP2 =  [g,    0,    0, 1;
             g, -2*e,    0, 1;
             g, -2*e, -2*d, 1;
             g,    0, -2*d, 1;
            -g,    0,    0, 1;
            -g, -2*e,    0, 1;
            -g, -2*e, -2*d, 1;
            -g,    0, -2*d, 1;
             0,   -e,   -d, 1];

    % SAP Rotation Matrices 

    R1 =  [1,           0,            0, 0;
           0,  cos(theta),  -sin(theta), 0;
           0,  sin(theta),   cos(theta), 0;
           0,           0,            0, 1];

    R2 =  [1,           0,            0, 0;
           0, cos(-theta), -sin(-theta), 0;
           0, sin(-theta),  cos(-theta), 0;
           0,           0,            0, 1];

    R1e = [1,          0,             0;
           0,  cos(theta),  -sin(theta);
           0,  sin(theta),   cos(theta)];

    R2e = [1,          0,             0;
           0, cos(-theta), -sin(-theta);
           0, sin(-theta),  cos(-theta)];

    for i = 1:9 % SAPs rotation

       SAP1(i,:) = (R1*SAP1(i,:)')' + [0,  b,  c, 0]; 
       SAP2(i,:) = (R2*SAP2(i,:)')' + [0, -b,  c, 0]; 

    end


    % Moment of Inertia Calculation

    Ixyz = [(1/3) * (b^2 + c^2),                   0,                   0;
                              0, (1/3) * (c^2 + a^2),                   0;
                              0,                   0, (1/3) * (a^2 + b^2)];
    Ixyz = Ixyz * M;

    Isap = [(1/3) * (e^2 + d^2),                   0,                    0;
                              0, (1/3) * (g^2 + d^2),                    0;
                              0,                   0, (1/3) * (g^2 + e^2)];

    Isap = Isap * Msap;

    Cg1  = ((SAP1(9,1:3) * Msap + SAP2(9,1:3) * Msap) / (M + 2 * Msap)); % Center of Gravity

    % Inertia Tensor of Target
    I   = (Ixyz + M * paxis(Cg1)) + (((R1e*Isap)*R1e') + Msap * paxis(SAP1(9,1:3) - Cg1)) + (((R2e*Isap)*R2e') + Msap * paxis(SAP2(9,1:3) - Cg1));


    mark = [1.5, 1, -2, 1;
            1.5, 0, -2, 1;
              0, 1, -2, 1;
              3, 0, -1, 1];
        
    if surfaces == 1
        mark(4,:) = [3, 0, -2, 1];
    end
          
    mks  = [mark(1,1:3)-mark(1,1:3) mark(2,1:3)-mark(1,1:3) mark(3,1:3)-mark(1,1:3) mark(4,1:3)-mark(1,1:3)]+1; csvwrite(ptSP,mks);

    cube = [a ,-b, c,1; a, b, c,1; a, b,-c,1; a,-b,-c,1;-a,-b, c,1;-a, b, c,1;-a, b,-c,1;-a,-b,-c,1;
            SAP1(1:8,:);
            SAP2(1:8,:);
            a ,-b,-c,1; a, b,-c,1;-a,-b,-c,1;-a, b,-c,1;    % Bottom:  25:28
            mark;                                           % Markers: 29:32
            0 , 0,-1,1; 1,0,0,1]';                          % Direct.: 33:34

    cube    = cube - [Cg1 0]';      % Brings CG to the origin

    pnp(n,cube,I,f,pos,vel,phi,omega,dts,dte,B,ST,IL,perr,point_animation,surfaces,estimation,prints,paths)

    if isfile(ptTS)     delete(ptTS);   end
    if isfile(ptTE)     delete(ptTE);   end
    if isfile(ptNP)     delete(ptNP);   end
        
    toc
    
end


%% FUNTIONS ---------------------------------------------------------------

function result = pnp(n,cube,I,f,pos,vel,phi0,omega0,dts,dte,B,ST,IL,perr,point_animation,surfaces,estimation,prints,paths)

    isMATLAB = false;
    LIC = license('inuse');
    for elem = 1:numel(LIC)
        envStr = LIC(elem).feature;
        if strcmpi(envStr,'matlab')
            isMATLAB = true;
            break
        end
    end
    
    cdc = paths{1}; ptME  = paths{2}; ptTE= paths{3}; ptTS  = paths{4}; ptSP = paths{5}; ptRE  = paths{6}; ptREf = paths{7}; ptNP = paths{8}; ptDT = paths{9}; ptRR = paths{10};
    
    if isfile(ptTS)     delete(ptTS);   end
    if isfile(ptTE)     delete(ptTE);   end
    if isfile(ptNP)     delete(ptNP);   end
    if isfile(ptRE)     delete(ptRE);   end
    if isfile(ptREf)    delete(ptREf);  end
    
    q01 = sin(phi0(3)/2) * cos(phi0(2)/2) * cos(phi0(1)/2) - cos(phi0(3)/2) * sin(phi0(2)/2) * sin(phi0(1)/2);
    q02 = cos(phi0(3)/2) * sin(phi0(2)/2) * cos(phi0(1)/2) + sin(phi0(3)/2) * cos(phi0(2)/2) * sin(phi0(1)/2);
    q03 = cos(phi0(3)/2) * cos(phi0(2)/2) * sin(phi0(1)/2) - sin(phi0(3)/2) * sin(phi0(2)/2) * cos(phi0(1)/2);
    q04 = cos(phi0(3)/2) * cos(phi0(2)/2) * cos(phi0(1)/2) + sin(phi0(3)/2) * sin(phi0(2)/2) * sin(phi0(1)/2);

    q0  = [q01;q02;q03;q04];
    
    R0 = [q01^2 - q02^2 - q03^2 + q04^2, 2 * (q01 * q02 + q03 * q04),2 * (q03 * q01 - q02 * q04);
          2 * (q01 * q02 - q03 * q04), q02^2 - q03^2 - q01^2 + q04^2,2 * (q02 * q03 + q01 * q04);
          2 * (q03 * q01 + q02 * q04), 2 * (q02 * q03 - q01 * q04),q03^2 - q01^2 - q02^2 + q04^2];
         
    cube3 = cube(1:3,:);
    
    cubelist = zeros(3,length(cube),n);
    
    cubelist(:,:,1) = cube3;
    
    for i = 1:n
    
        R(:,:,i) = [1 0 0;
                    0 1 0;
                    0 0 1];
    
        omega(:,i) = omega0;
        
    end
    
    R(:,:,1) = R0*R(:,:,1);
    
    q       = [zeros(3,n);ones(1,n)];
    q(:,1)  = q0/norm(q0); 
    
    t       = [];
    te      = [];
    
    O       = [0,0,0,1];
    
    Pr      = [f,0,0,0;
               0,f,0,0;
               0,0,1,0];
    
    estplot = [];
    ansplot = [];
    
    visible_previous    = 0;
    estcount            = 0;
    
    Lest   = NaN(3,n);
    oest   = NaN(3,n);
    Lans   = NaN(3,n);
    Iest   = NaN(3,3,n);
    Iest11 = NaN(1,n);
    Iest22 = NaN(1,n);
    Iest33 = NaN(1,n);
    Iest12 = NaN(1,n);
    Iest13 = NaN(1,n);
    Iest23 = NaN(1,n);
    
    if ~isMATLAB
        
        export = [];
    
        estimfigure = figure('position',[50 50 1600 900]);
        subplot(3,1,1)
        xlim([0 n*dts])
        grid on
        ani111 = animatedlineo('Color',[1         0         0],'LineWidth',2);
        ani112 = animatedlineo('Color',[0         0    1.0000],'LineWidth',2);
        ani113 = animatedlineo('Color',[0    0.6000         0],'LineWidth',2);
        if estimation == 1
            ani121 = animatedlineo('Marker','o','MarkerFaceColor',[1.0000    0.7020    0.7020],'Color',[1.0000    0.7020    0.7020],'LineWidth',1);
            ani122 = animatedlineo('Marker','o','MarkerFaceColor',[0.7020    0.7020    1.0000],'Color',[0.7020    0.7020    1.0000],'LineWidth',1);
            ani123 = animatedlineo('Marker','o','MarkerFaceColor',[0.3020    1.0000    0.3020],'Color',[0.3020    1.0000    0.3020],'LineWidth',1);
            legend({'L_{true_{x}}','L_{true_{y}}','L_{true_{z}}','L_{est_{x}}','L_{est_{y}}','L_{est_{z}}'},...
                    'NumColumns',2)
            h1 = get(gca,'Children');
            set(gca,'Children',[h1(4) h1(5) h1(6) h1(1) h1(2) h1(3)])
        else
            legend({'L_{true_{x}}','L_{true_{y}}','L_{true_{z}}'},...
                    'NumColumns',1)
            h1 = get(gca,'Children');
            set(gca,'Children',[h1(1) h1(2) h1(3)])
        end

        subplot(3,1,2)
        xlim([0 n*dts])
        grid on
        ani211 = animatedlineo('Color',[1.0000    0         0],'LineWidth',2);
        ani212 = animatedlineo('Color',[0         0    1.0000],'LineWidth',2);
        ani213 = animatedlineo('Color',[0    0.6000         0],'LineWidth',2);
        if estimation == 1
            ani221 = animatedlineo('Marker','o','MarkerFaceColor',[1.0000    0.7020    0.7020],'Color',[1.0000    0.7020    0.7020],'LineWidth',1);
            ani222 = animatedlineo('Marker','o','MarkerFaceColor',[0.7020    0.7020    1.0000],'Color',[0.7020    0.7020    1.0000],'LineWidth',1);
            ani223 = animatedlineo('Marker','o','MarkerFaceColor',[0.3020    1.0000    0.3020],'Color',[0.3020    1.0000    0.3020],'LineWidth',1);
            legend({'\omega_{true_{x}}','\omega_{true_{y}}','\omega_{true_{z}}','\omega_{est_{x}}','\omega_{est_{y}}','\omega_{est_{z}}'},...
                    'NumColumns',2)
            h2 = get(gca,'Children');
            set(gca,'Children',[h2(4) h2(5) h2(6) h2(1) h2(2) h2(3)])
        else
            legend({'\omega_{true_{x}}','\omega_{true_{y}}','\omega_{true_{z}}'},...
                    'NumColumns',1)
            h2 = get(gca,'Children');
            set(gca,'Children',[h2(1) h2(2) h2(3)])
        end

        subplot(3,1,3)
        xlim([0 n*dts])
        grid on
        ani311 = animatedlineo('Color',[1.0000       0         0],'LineWidth',2);
        ani312 = animatedlineo('Color',[0            0    1.0000],'LineWidth',2);
        ani313 = animatedlineo('Color',[0       0.6000         0],'LineWidth',2);
        ani314 = animatedlineo('Color',[0.9020  0.6745         0],'LineWidth',2);
        ani315 = animatedlineo('Color',[0.7490       0    1.0000],'LineWidth',2);
        ani316 = animatedlineo('Color',[0       0.6000    0.8000],'LineWidth',2);
        if estimation == 1
            ani321 = animatedlineo('Marker','o','MarkerFaceColor',[1.0000    0.7020    0.7020],'Color',[1.0000    0.7020    0.7020],'LineWidth',1);
            ani322 = animatedlineo('Marker','o','MarkerFaceColor',[0.7020    0.7020    1.0000],'Color',[0.7020    0.7020    1.0000],'LineWidth',1);
            ani323 = animatedlineo('Marker','o','MarkerFaceColor',[0.3020    1.0000    0.3020],'Color',[0.3020    1.0000    0.3020],'LineWidth',1);
            ani324 = animatedlineo('Marker','o','MarkerFaceColor',[1.0000    0.8510    0.4000],'Color',[1.0000    0.8510    0.4000],'LineWidth',1);
            ani325 = animatedlineo('Marker','o','MarkerFaceColor',[0.9255    0.7020    1.0000],'Color',[0.9255    0.7020    1.0000],'LineWidth',1);
            ani326 = animatedlineo('Marker','o','MarkerFaceColor',[0.3020    0.8235    1.0000],'Color',[0.3020    0.8235    1.0000],'LineWidth',1);
            legend({'I_{true_{xx}}','I_{true_{yy}}','I_{true_{zz}}','I_{true_{xy}}','I_{true_{xz}}','I_{true_{yz}}',...
                    'I_{est_{xx}}', 'I_{est_{yy}}', 'I_{est_{zz}}', 'I_{est_{xy}}', 'I_{est_{xz}}', 'I_{est_{yz}}'},...
                    'NumColumns',2)
            h3 = get(gca,'Children');
            set(gca,'Children',[h3(7) h3(8) h3(9) h3(10) h3(11) h3(12) h3(1) h3(2) h3(3) h3(4) h3(5) h3(6)])
        else
            legend({'I_{true_{xx}}','I_{true_{yy}}','I_{true_{zz}}','I_{true_{xy}}','I_{true_{xz}}','I_{true_{yz}}'},...
                    'NumColumns',1)
            h3 = get(gca,'Children');
            set(gca,'Children',[h3(1) h3(2) h3(3) h3(4) h3(5) h3(6)])
        end

        if point_animation == 1
            motion = figure;
            ax = axes;
            axis equal
            set (ax,'XLim',[-f/2 f/2],'YLim',[-f/2 f/2])
            xlabel("X, pixels")
            ylabel("Y, pixels")
            grid on
            hold on

            p1      = plot(0,0,'o','MarkerFaceColor',[0.0000 0.4470 0.7410],'MarkerEdgeColor',[0.0000 0.4470 0.7410]);
            p2      = plot(0,0,'o','MarkerFaceColor',[0.8500 0.3250 0.0980],'MarkerEdgeColor',[0.8500 0.3250 0.0980]);
            p3      = plot(0,0,'o','MarkerFaceColor',[0.9290 0.6940 0.1250],'MarkerEdgeColor',[0.9290 0.6940 0.1250]);
            p4      = plot(0,0,'o','MarkerFaceColor',[0.4940 0.1840 0.5560],'MarkerEdgeColor',[0.4940 0.1840 0.5560]);

            p11     = plot(0,0,'o','MarkerFaceColor','k','MarkerEdgeColor','k');
            p12     = plot(0,0,'o','MarkerFaceColor','k','MarkerEdgeColor','k');
            p13     = plot(0,0,'o','MarkerFaceColor','k','MarkerEdgeColor','k');
            p14     = plot(0,0,'o','MarkerFaceColor','k','MarkerEdgeColor','k');
            p15     = plot(0,0,'o','MarkerFaceColor','k','MarkerEdgeColor','k');
            p16     = plot(0,0,'o','MarkerFaceColor','k','MarkerEdgeColor','k');
            l12     = plot([0 0],[0 0],'k');
            l23     = plot([0 0],[0 0],'k');
            l34     = plot([0 0],[0 0],'k');
            l41     = plot([0 0],[0 0],'k');
            l35     = plot([0 0],[0 0],'k');
            l56     = plot([0 0],[0 0],'k');
            l64     = plot([0 0],[0 0],'k');

            legend({'Marker 1','Marker 2','Marker 3','Marker 4',' ',' ',' ',' '},...
                    'NumColumns',1)
            title('Projection of Markers 1, 2, 3 and 4, and Edges of Bottom and Front Panels')
            h4 = get(gca,'Children');
            set(gca,'Children',[h4(5) h4(6) h4(7) h4(8) h4(9) h4(10) h4(11) h4(12) h4(13) h4(14) h4(15) h4(16) h4(17) h4(1) h4(2) h4(3) h4(4)])
        end
    
    else
        
        export = ["t", "Ixx_true", "Iyy_true", "Izz_true", "Ixy_true", "Ixz_true", "Iyz_true",...
                  "omega1_true", "omega2_true", "omega3_true", "L1_true", "L2_true", "L3_true",...
                  "Ixx_est",  "Iyy_est",  "Izz_est",  "Ixy_est",  "Ixz_est",  "Iyz_est",...
                  "omega1_est",  "omega2_est",  "omega3_est",  "L1_est",  "L2_est",  "L3_est",...
                  "Markers_hidden?","Estimator_restart?","Marker1_x","Marker1_y","Marker1_z",...
                  "Marker2_x","Marker2_y","Marker2_z","Marker3_x","Marker3_y","Marker3_z",...
                  "Marker4_x","Marker4_y","Marker4_z","DCM11","DCM12","DCM13","DCM21",...
                  "DCM22","DCM23","DCM31","DCM32","DCM33"];
              
        estimfigure = figure('position',[50 50 1600 900]);
        set(findall(estimfigure,'-property','FontSize'),'FontSize',13)
        subplot(3,1,1)
        if perr == 0
        %title({['B = ',num2str(B),' | Inputs are float | Error OFF'];['3 Markers on one surface, 1 Marker on another'];[' ']})
        title({['True vs. Estimated Parameters'];[' ']})
        else
        %title({['B = ',num2str(B),' | Inputs are float | Error ON']; ['3 Markers on one surface, 1 Marker on another'];[' ']})
        title({['True vs. Estimated Parameters'];[' ']})
        end
        xlim([0 n*dts])
        ylabel("Unitary Angular Momentum")
        grid on
        ani111 = animatedline('Color',[1 0 0],'LineWidth',3);
        ani112 = animatedline('Color',[0         0    1.0000],'LineWidth',3);
        ani113 = animatedline('Color',[0    0.6000         0],'LineWidth',3);
        if estimation == 1
            ani121 = animatedline('Marker','o','MarkerFaceColor',[1.0000    0.7020    0.7020],'Color',[1.0000    0.7020    0.7020],'LineWidth',2);
            ani122 = animatedline('Marker','o','MarkerFaceColor',[0.7020    0.7020    1.0000],'Color',[0.7020    0.7020    1.0000],'LineWidth',2);
            ani123 = animatedline('Marker','o','MarkerFaceColor',[0.3020    1.0000    0.3020],'Color',[0.3020    1.0000    0.3020],'LineWidth',2);
            legend({'L_{true_{x}}','L_{true_{y}}','L_{true_{z}}','L_{est_{x}}','L_{est_{y}}','L_{est_{z}}'},...
                    'NumColumns',2)
            h1 = get(gca,'Children');
            set(gca,'Children',[h1(4) h1(5) h1(6) h1(1) h1(2) h1(3)])
        else
            legend({'L_{true_{x}}','L_{true_{y}}','L_{true_{z}}'},...
                    'NumColumns',1)
            h1 = get(gca,'Children');
            set(gca,'Children',[h1(1) h1(2) h1(3)])
        end

        subplot(3,1,2)
        xlim([0 n*dts])
        ylabel("Angular Velocity, rad sec-1")
        grid on
        ani211 = animatedline('Color',[1.0000 0 0],'LineWidth',3);
        ani212 = animatedline('Color',[0         0    1.0000],'LineWidth',3);
        ani213 = animatedline('Color',[0    0.6000         0],'LineWidth',3);
        if estimation == 1
            ani221 = animatedline('Marker','o','MarkerFaceColor',[1.0000    0.7020    0.7020],'Color',[1.0000    0.7020    0.7020],'LineWidth',2);
            ani222 = animatedline('Marker','o','MarkerFaceColor',[0.7020    0.7020    1.0000],'Color',[0.7020    0.7020    1.0000],'LineWidth',2);
            ani223 = animatedline('Marker','o','MarkerFaceColor',[0.3020    1.0000    0.3020],'Color',[0.3020    1.0000    0.3020],'LineWidth',2);
            legend({'\omega_{true_{x}}','\omega_{true_{y}}','\omega_{true_{z}}','\omega_{est_{x}}','\omega_{est_{y}}','\omega_{est_{z}}'},...
                    'NumColumns',2)
            h2 = get(gca,'Children');
            set(gca,'Children',[h2(4) h2(5) h2(6) h2(1) h2(2) h2(3)])
        else
            legend({'\omega_{true_{x}}','\omega_{true_{y}}','\omega_{true_{z}}'},...
                    'NumColumns',1)
            h2 = get(gca,'Children');
            set(gca,'Children',[h2(1) h2(2) h2(3)])
        end

        subplot(3,1,3)
        xlim([0 n*dts])
        ylabel("Moment of Inertia")
        xlabel("Time, sec")
        grid on
        ani311 = animatedline('Color',[1.0000         0         0],'LineWidth',3);
        ani312 = animatedline('Color',[0         0    1.0000],'LineWidth',3);
        ani313 = animatedline('Color',[0    0.6000         0],'LineWidth',3);
        ani314 = animatedline('Color',[0.9020    0.6745         0],'LineWidth',3);
        ani315 = animatedline('Color',[0.7490         0    1.0000],'LineWidth',3);
        ani316 = animatedline('Color',[0    0.6000    0.8000],'LineWidth',3);
        if estimation == 1
            ani321 = animatedline('Marker','o','MarkerFaceColor',[1.0000    0.7020    0.7020],'Color',[1.0000    0.7020    0.7020],'LineWidth',2);
            ani322 = animatedline('Marker','o','MarkerFaceColor',[0.7020    0.7020    1.0000],'Color',[0.7020    0.7020    1.0000],'LineWidth',2);
            ani323 = animatedline('Marker','o','MarkerFaceColor',[0.3020    1.0000    0.3020],'Color',[0.3020    1.0000    0.3020],'LineWidth',2);
            ani324 = animatedline('Marker','o','MarkerFaceColor',[1.0000    0.8510    0.4000],'Color',[1.0000    0.8510    0.4000],'LineWidth',2);
            ani325 = animatedline('Marker','o','MarkerFaceColor',[0.9255    0.7020    1.0000],'Color',[0.9255    0.7020    1.0000],'LineWidth',2);
            ani326 = animatedline('Marker','o','MarkerFaceColor',[0.3020    0.8235    1.0000],'Color',[0.3020    0.8235    1.0000],'LineWidth',2);
            legend({'I_{true_{xx}}','I_{true_{yy}}','I_{true_{zz}}','I_{true_{xy}}','I_{true_{xz}}','I_{true_{yz}}',...
                    'I_{est_{xx}}', 'I_{est_{yy}}', 'I_{est_{zz}}', 'I_{est_{xy}}', 'I_{est_{xz}}', 'I_{est_{yz}}'},...
                    'NumColumns',2)
            h3 = get(gca,'Children');
            set(gca,'Children',[h3(7) h3(8) h3(9) h3(10) h3(11) h3(12) h3(1) h3(2) h3(3) h3(4) h3(5) h3(6)])
        else
            legend({'I_{true_{xx}}','I_{true_{yy}}','I_{true_{zz}}','I_{true_{xy}}','I_{true_{xz}}','I_{true_{yz}}'},...
                    'NumColumns',1)
            h3 = get(gca,'Children');
            set(gca,'Children',[h3(1) h3(2) h3(3) h3(4) h3(5) h3(6)])
        end
        set(findall(estimation,'-property','FontSize'),'FontSize',13)

        if point_animation == 1
            motion = figure;
            ax = axes;
            axis equal
            set (ax,'XLim',[-f/2 f/2],'YLim',[-f/2 f/2])
            title({["Animation of the Markers' Projections"];['and Front and Bottom Panels'];['']})
            xlabel("X, pixels")
            ylabel("Y, pixels")
            grid on
            hold on

            p1      = plot(0,0,'o','MarkerFaceColor','#0072BD','MarkerEdgeColor','#0072BD');
            p2      = plot(0,0,'o','MarkerFaceColor','#D95319','MarkerEdgeColor','#D95319');
            p3      = plot(0,0,'o','MarkerFaceColor','#EDB120','MarkerEdgeColor','#EDB120');
            p4      = plot(0,0,'o','MarkerFaceColor','#7E2F8E','MarkerEdgeColor','#7E2F8E');

            p11     = plot(0,0,'o','MarkerFaceColor','k','MarkerEdgeColor','k');
            p12     = plot(0,0,'o','MarkerFaceColor','k','MarkerEdgeColor','k');
            p13     = plot(0,0,'o','MarkerFaceColor','k','MarkerEdgeColor','k');
            p14     = plot(0,0,'o','MarkerFaceColor','k','MarkerEdgeColor','k');
            p15     = plot(0,0,'o','MarkerFaceColor','k','MarkerEdgeColor','k');
            p16     = plot(0,0,'o','MarkerFaceColor','k','MarkerEdgeColor','k');
            l12     = plot([0 0],[0 0],'k');
            l23     = plot([0 0],[0 0],'k');
            l34     = plot([0 0],[0 0],'k');
            l41     = plot([0 0],[0 0],'k');
            l35     = plot([0 0],[0 0],'k');
            l56     = plot([0 0],[0 0],'k');
            l64     = plot([0 0],[0 0],'k');
            
            legend('Marker 1','Marker 2','Marker 3','Marker 4')
            
            set(findall(motion,'-property','FontSize'),'FontSize',13)
            h4 = get(gca,'Children');
            set(gca,'Children',[h4(5) h4(6) h4(7) h4(8) h4(9) h4(10) h4(11) h4(12) h4(13) h4(14) h4(15) h4(16) h4(17) h4(1) h4(2) h4(3) h4(4)])
        end
              
    end
    
    savetime = 50;
    
    for i = 1:(n-1)
        
        reset = 0;
        
        if i == 1
            counter     = dte/dts;
            reset       = 1;
            
            if isfile(ptDT) delete(ptDT); end
            if isfile(ptNP) delete(ptNP); end
            mkdir(ptDT)
            
        end
        
        currstep = counter*dts;
        
        t = [t;(dts*(i-1))];
        
        % 4TH ORDER RUNGE KUTTA METHOD - Integration of Attitude Dynamics
       
        k1 = -(I\cross(omega(:,i),(I*omega(:,i))));
        k2 = -(I\cross((omega(:,i) + k1/2 *dts),(I*(omega(:,i) + k1/2 *dts))));
        k3 = -(I\cross((omega(:,i) + k2/2 *dts),(I*(omega(:,i) + k2/2 *dts))));
        k4 = -(I\cross((omega(:,i) + k3   *dts),(I*(omega(:,i) + k3   *dts))));
        
        omega(:,(i+1)) = omega(:,i) + (k1 + 2*k2 + 2*k3 + k4)/6 * dts;
  
        osk = [            0, omega(3,i+1),-omega(2,i+1), omega(1,i+1);
               -omega(3,i+1),            0, omega(1,i+1), omega(2,i+1);
                omega(2,i+1),-omega(1,i+1),            0, omega(3,i+1);
               -omega(1,i+1),-omega(2,i+1),-omega(3,i+1),            0];
                  
        kq1 = (1/2)* osk * (q(:,i));                  
        kq2 = (1/2)* osk * (q(:,i) + kq1/2 *dts);                  
        kq3 = (1/2)* osk * (q(:,i) + kq2/2 *dts);                  
        kq4 = (1/2)* osk * (q(:,i) + kq3   *dts);
        
        aux = q(:,i) + (kq1 + 2*kq2 + 2*kq3 + kq4)/6 * dts;
        
        q(:,(i+1)) = aux/norm(aux);
        
        R(:,:,(i+1)) = [(q(1,i+1)^2 - q(2,i+1)^2 - q(3,i+1)^2 + q(4,i+1)^2),      (2 * (q(1,i+1)*q(2,i+1) + q(3,i+1)*q(4,i+1))),      (2 * (q(3,i+1)*q(1,i+1) - q(2,i+1)*q(4,i+1)));
                              (2 * (q(1,i+1)*q(2,i+1) - q(3,i+1)*q(4,i+1))),(q(2,i+1)^2 - q(3,i+1)^2 - q(1,i+1)^2 + q(4,i+1)^2),      (2 * (q(2,i+1)*q(3,i+1) + q(1,i+1)*q(4,i+1)));
                              (2 * (q(3,i+1)*q(1,i+1) + q(2,i+1)*q(4,i+1))),      (2 * (q(2,i+1)*q(3,i+1) - q(1,i+1)*q(4,i+1))),(q(3,i+1)^2 - q(1,i+1)^2 - q(2,i+1)^2 + q(4,i+1)^2)]';
        
        if i == 1
            cubelist(:,:,i) = R(:,:,i)*cube3;
        end
        
        cubelist(:,:,(i+1)) = R(:,:,(i+1))*cube3;

        T(:,i)      = [pos(1) + vel(1) * (i-1) * dts;
                       pos(2) + vel(2) * (i-1) * dts;
                       pos(3) + vel(3) * (i-1) * dts];
                   
        cbt(:,:,i)  = cubelist(:,:,i) + T(:,i);

        aux         = [R(:,:,i),T(:,i)];
        RT(:,:,i)   = [aux;O];
                    
        xy(:,:,i)   = RT(:,:,i) * cube;
        xyz(:,:,i)  = Pr * xy(:,:,i);
        mk          = xy(1:3,29:32,i);
        
        err1        = randn(1,34)*perr;
        err2        = randn(1,34)*perr;
        
        projection(:,:,i) = [xyz(1,:,i)./xyz(3,:,i);xyz(2,:,i)./xyz(3,:,i)];
        projection(:,:,i) = [projection(1,:,i) + err1;
                             projection(2,:,i) + err2];
        storedproj(i)     = projection(1,29,i);
                          
        n1(:,i)     = cubelist(:,33,i);
        n1a(i)      = dot(mk(:,2),n1(:,i))/(norm(mk(:,2))*(norm(n1(:,i))));
        
        if surfaces == 2
            n2(:,i)     = cubelist(:,34,i);
            n2a(i)      = dot(mk(:,4),n2(:,i))/(norm(mk(:,4))*(norm(n2(:,i))));
        elseif surfaces == 1
            n2a(i)      = -1;
        end
        
        if currstep >= dte-0.000001
            
            if estcount/savetime >= 0.3 || t(i) < savetime
                clc
                fprintf("\n\nt = %.f sec\n\n", t(i))
            end
            counter = 0;
            
            visible_flag = 0;
            if n1a(i) >= 0 || n2a(i) >= 0
                visible_flag = 1;
                projection(1,29,i) = 2000;   
            end
            
            if estimation == 1
                input1  = [projection(:,29,i)', projection(:,30,i)', projection(:,31,i)', projection(:,32,i)', IL, ST, t(i), dte, B, reset];
                 
                if prints == 1
                    fprintf('\n\nInput sent to estimator: %s \n',num2str(input1))
                end

                csvwrite(ptTE,input1);

                if ismac

                    command = sprintf('"%s" "%s" "%s" "%s" "%s"', ptME, ptTE, cdc, ptTS, ptSP);
                    system(command);

                elseif ispc

                    command = sprintf('"%s" "%s" "%s" "%s" "%s"', ptME, ptTE, cdc, ptTS, ptSP);
                    system(command);

                end
            end
            
            estcount = estcount + 1;
            
            if reset == 0
                
                if estimation == 1
                    partial = csvread(ptTS);
                    
                    Lest(:,i)    = partial(1,:)';
                    oest(:,i)    = partial(2,:)';
                    Iest(:,:,i)  = partial(3:5,:);
                    Rest(:,:,i)  = partial(6:9,:)';
                    DCM(:,:,i)   = partial(10:12,:);
                    Iest11(i)    = Iest(1,1,i);
                    Iest22(i)    = Iest(2,2,i);
                    Iest33(i)    = Iest(3,3,i);
                    Iest12(i)    = Iest(1,2,i);
                    Iest13(i)    = Iest(1,3,i);
                    Iest23(i)    = Iest(2,3,i);
                else
                    Lest(:,i)    = NaN(3,1);
                    oest(:,i)    = NaN(3,1);
                    Iest(:,:,i)  = NaN(3,3);
                    Rest(:,:,i)  = NaN(3,4);
                    DCM(:,:,i)   = NaN(3,3);
                    Iest11(i)    = Iest(1,1,i);
                    Iest22(i)    = Iest(2,2,i);
                    Iest33(i)    = Iest(3,3,i);
                    Iest12(i)    = Iest(1,2,i);
                    Iest13(i)    = Iest(1,3,i);
                    Iest23(i)    = Iest(2,3,i);
                end
                
                Lansa = R(:,:,i)*(I*omega(:,i));
                nLans= norm(Lansa);
                Lans(:,i) = Lansa/nLans;

                Iansa(:,:,i) = I./nLans;
                
                Ians11(i)  = Iansa(1,1,i);
                Ians22(i)  = Iansa(2,2,i);
                Ians33(i)  = Iansa(3,3,i);
                Ians12(i)  = Iansa(1,2,i);
                Ians13(i)  = Iansa(1,3,i);
                Ians23(i)  = Iansa(2,3,i);

                export = [export;[t(i), Ians11(i), Ians22(i), Ians33(i),...
                          Ians12(i), Ians13(i), Ians23(i), omega(:,i)',...
                          Lans(:,i)', Iest11(i), Iest22(i), Iest33(i),...
                          Iest12(i), Iest13(i), Iest23(i), oest(:,i)',...
                          Lest(:,i)', visible_flag, reset, Rest(:,1,i)',...
                          Rest(:,2,i)',Rest(:,3,i)',Rest(:,4,i)',...
                          DCM(1,:,i), DCM(2,:,i), DCM(3,:,i)]];
                     
                
                if estcount/savetime >= 0.99
                    estcount = 0;
                    if ~isMATLAB
                        csvwrite(ptRE,export);
                    else
                       writematrix(export,ptRE); 
                    end
                    clc
                    fprintf("\n\nt = %.f sec | Results saved as %s\n\n", t(i), ptRE)
                end
                
                ansplot = [ansplot,i];
                
                if visible_flag == 0 && visible_previous == 0
                    
                    estplot = [estplot,i];
                    
                end
                
                if point_animation == 1
                    if ~isMATLAB
                        clf(motion)
                        ax = axes;
                        hold on
                        grid on
                        axis equal
                        set (ax,'XLim',[-f/2 f/2],'YLim',[-f/2 f/2])
                        set(findall(motion,'-property','FontSize'),'FontSize',13)
                        xlabel("X, pixels")
                        ylabel("Y, pixels")
                        p11 = plot(projection(1,3,i),projection(2,3,i),'o','MarkerFaceColor','k','MarkerEdgeColor','k');
                        p12 = plot(projection(1,4,i),projection(2,4,i),'o','MarkerFaceColor','k','MarkerEdgeColor','k');
                        p13 = plot(projection(1,7,i),projection(2,7,i),'o','MarkerFaceColor','k','MarkerEdgeColor','k');
                        p14 = plot(projection(1,8,i),projection(2,8,i),'o','MarkerFaceColor','k','MarkerEdgeColor','k');
                        p15 = plot(projection(1,1,i),projection(2,1,i),'o','MarkerFaceColor','k','MarkerEdgeColor','k');
                        p16 = plot(projection(1,2,i),projection(2,2,i),'o','MarkerFaceColor','k','MarkerEdgeColor','k');
                        l12         = plot([projection(1,3,i) projection(1,4,i)],[projection(2,3,i) projection(2,4,i)],'k');
                        l23         = plot([projection(1,4,i) projection(1,8,i)],[projection(2,4,i) projection(2,8,i)],'k');
                        l34         = plot([projection(1,8,i) projection(1,7,i)],[projection(2,8,i) projection(2,7,i)],'k');
                        l41         = plot([projection(1,7,i) projection(1,3,i)],[projection(2,7,i) projection(2,3,i)],'k');
                        l35         = plot([projection(1,3,i) projection(1,2,i)],[projection(2,3,i) projection(2,2,i)],'k');
                        l56         = plot([projection(1,1,i) projection(1,2,i)],[projection(2,1,i) projection(2,2,i)],'k');
                        l64         = plot([projection(1,1,i) projection(1,4,i)],[projection(2,1,i) projection(2,4,i)],'k');
                        if visible_flag == 0
                            p1   = plot(storedproj(i)     ,projection(2,29,i),'o','MarkerFaceColor',[0.0000 0.4470 0.7410],'MarkerEdgeColor',[0.0000 0.4470 0.7410]);
                            p2   = plot(projection(1,30,i),projection(2,30,i),'o','MarkerFaceColor',[0.8500 0.3250 0.0980],'MarkerEdgeColor',[0.8500 0.3250 0.0980]);
                            p3   = plot(projection(1,31,i),projection(2,31,i),'o','MarkerFaceColor',[0.9290 0.6940 0.1250],'MarkerEdgeColor',[0.9290 0.6940 0.1250]);
                            p4   = plot(projection(1,32,i),projection(2,32,i),'o','MarkerFaceColor',[0.4940 0.1840 0.5560],'MarkerEdgeColor',[0.4940 0.1840 0.5560]);
                        else
                            p1   = plot(storedproj(i)     ,projection(2,29,i),'o','MarkerFaceColor',[0.7843 0.7843 0.7843],'MarkerEdgeColor',[0.0000 0.4470 0.7410]);
                            p2   = plot(projection(1,30,i),projection(2,30,i),'o','MarkerFaceColor',[0.7843 0.7843 0.7843],'MarkerEdgeColor',[0.8500 0.3250 0.0980]);
                            p3   = plot(projection(1,31,i),projection(2,31,i),'o','MarkerFaceColor',[0.7843 0.7843 0.7843],'MarkerEdgeColor',[0.9290 0.6940 0.1250]);
                            p4   = plot(projection(1,32,i),projection(2,32,i),'o','MarkerFaceColor',[0.7843 0.7843 0.7843],'MarkerEdgeColor',[0.4940 0.1840 0.5560]);
                        end
                    else
                        p11.XData = projection(1,3,i); p11.YData = projection(2,3,i);
                        p12.XData = projection(1,4,i); p12.YData = projection(2,4,i);
                        p13.XData = projection(1,7,i); p13.YData = projection(2,7,i);
                        p14.XData = projection(1,8,i); p14.YData = projection(2,8,i);
                        p15.XData = projection(1,1,i); p15.YData = projection(2,1,i);
                        p16.XData = projection(1,2,i); p16.YData = projection(2,2,i);
                        l12.XData = [projection(1,3,i) projection(1,4,i)]; l12.YData = [projection(2,3,i) projection(2,4,i)];
                        l23.XData = [projection(1,4,i) projection(1,8,i)]; l23.YData = [projection(2,4,i) projection(2,8,i)];
                        l34.XData = [projection(1,8,i) projection(1,7,i)]; l34.YData = [projection(2,8,i) projection(2,7,i)];
                        l41.XData = [projection(1,7,i) projection(1,3,i)]; l41.YData = [projection(2,7,i) projection(2,3,i)];
                        l23.XData = [projection(1,4,i) projection(1,8,i)]; l23.YData = [projection(2,4,i) projection(2,8,i)];
                        l35.XData = [projection(1,3,i) projection(1,2,i)]; l35.YData = [projection(2,3,i) projection(2,2,i)];
                        l56.XData = [projection(1,1,i) projection(1,2,i)]; l56.YData = [projection(2,1,i) projection(2,2,i)];
                        l64.XData = [projection(1,1,i) projection(1,4,i)]; l64.YData = [projection(2,1,i) projection(2,4,i)];
                        if visible_flag == 0
                            p1.XData = storedproj(i);      p1.YData = projection(2,29,i); p1.MarkerFaceColor = '#0072BD';
                            p2.XData = projection(1,30,i); p2.YData = projection(2,30,i); p2.MarkerFaceColor = '#D95319';
                            p3.XData = projection(1,31,i); p3.YData = projection(2,31,i); p3.MarkerFaceColor = '#EDB120';
                            p4.XData = projection(1,32,i); p4.YData = projection(2,32,i); p4.MarkerFaceColor = '#7E2F8E';
                        else
                            p1.XData = storedproj(i);      p1.YData = projection(2,29,i); p1.MarkerFaceColor = [0.7843 0.7843 0.7843];
                            p2.XData = projection(1,30,i); p2.YData = projection(2,30,i); p2.MarkerFaceColor = [0.7843 0.7843 0.7843];
                            p3.XData = projection(1,31,i); p3.YData = projection(2,31,i); p3.MarkerFaceColor = [0.7843 0.7843 0.7843];
                            p4.XData = projection(1,32,i); p4.YData = projection(2,32,i); p4.MarkerFaceColor = [0.7843 0.7843 0.7843];
                        end
                    end
                end
                addpoints(ani111,t(i),Lans(1,i));
                addpoints(ani112,t(i),Lans(2,i));
                addpoints(ani113,t(i),Lans(3,i));
                if visible_flag == 0 && visible_previous == 0
                    addpoints(ani121,t(i),Lest(1,i))
                    addpoints(ani122,t(i),Lest(2,i))
                    addpoints(ani123,t(i),Lest(3,i))
                else
                    addpoints(ani121,t(i),NaN)
                    addpoints(ani122,t(i),NaN)
                    addpoints(ani123,t(i),NaN)
                end
                
                addpoints(ani211,t(i),omega(1,i));
                addpoints(ani212,t(i),omega(2,i));
                addpoints(ani213,t(i),omega(3,i));
                if visible_flag == 0 && visible_previous == 0
                    addpoints(ani221,t(i),oest(1,i))
                    addpoints(ani222,t(i),oest(2,i))
                    addpoints(ani223,t(i),oest(3,i))
                else
                    addpoints(ani221,t(i),NaN)
                    addpoints(ani222,t(i),NaN)
                    addpoints(ani223,t(i),NaN)
                end
                
                addpoints(ani311,t(i),Ians11(i));
                addpoints(ani312,t(i),Ians22(i));
                addpoints(ani313,t(i),Ians33(i));
                addpoints(ani314,t(i),Ians12(i));
                addpoints(ani315,t(i),Ians13(i));
                addpoints(ani316,t(i),Ians23(i));
                if visible_flag == 0 && visible_previous == 0
                    addpoints(ani321,t(i),Iest11(i))
                    addpoints(ani322,t(i),Iest22(i))
                    addpoints(ani323,t(i),Iest33(i))
                    addpoints(ani324,t(i),Iest12(i))
                    addpoints(ani325,t(i),Iest13(i))
                    addpoints(ani326,t(i),Iest23(i))
                else
                    addpoints(ani321,t(i),NaN)
                    addpoints(ani322,t(i),NaN)
                    addpoints(ani323,t(i),NaN)
                    addpoints(ani324,t(i),NaN)
                    addpoints(ani325,t(i),NaN)
                    addpoints(ani326,t(i),NaN)
                end
                refresh
                pause(1/120)
                
                if visible_flag == 0 && visible_previous == 1
                    visible_previous = 0;
                end
                if visible_flag == 1
                    visible_previous = 1;
                end
                
            end
            
        end
            
       
        counter = counter+1;
        
    end
    
    if ~isMATLAB
        csvwrite(ptRE,export);
    else
        writematrix(export,ptRE);
    end
    saveas(estimfigure,ptREf);
    clc
    fprintf("\n\nSimulation finalized at t = %.f sec | Results saved as %s and %s\n\n", t(i), ptRE, ptREf)
    
end

function result = paxis(input)

    x = input(1); y = input(2); z = input(3); 
    
    result = [y^2 + z^2, -y*x, -z*x;
             -x*y, x^2 + y^2, -z*y;
             -x*z,-y*z,x^2+y^2];
    
end
